@@ -1,40 +1,41 @@
 /*
-    C-Dogs SDL
-    A port of the legendary (and fun) action/arcade cdogs.
-    Copyright (c) 2014, 2019 Cong Xu
-    All rights reserved.
+	C-Dogs SDL
+	A port of the legendary (and fun) action/arcade cdogs.
+	Copyright (c) 2014, 2019-2021 Cong Xu
+	All rights reserved.
 
-    Redistribution and use in source and binary forms, with or without
-    modification, are permitted provided that the following conditions are met:
+	Redistribution and use in source and binary forms, with or without
+	modification, are permitted provided that the following conditions are met:
 
-    Redistributions of source code must retain the above copyright notice, this
-    list of conditions and the following disclaimer.
-    Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
+	Redistributions of source code must retain the above copyright notice, this
+	list of conditions and the following disclaimer.
+	Redistributions in binary form must reproduce the above copyright notice,
+	this list of conditions and the following disclaimer in the documentation
+	and/or other materials provided with the distribution.
 
-    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-    AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-    ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-    LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-    CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-    SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-    INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-    CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-    ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-    POSSIBILITY OF SUCH DAMAGE.
+	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+	CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+	POSSIBILITY OF SUCH DAMAGE.
 */
 #include "editor_brush.h"
 
 #include <assert.h>
 
+#include <SDL_image.h>
+
 #include <cdogs/algorithms.h>
 #include <cdogs/map.h>
 #include <cdogs/map_build.h>
 #include <cdogs/mission_convert.h>
 
-
 void EditorBrushInit(EditorBrush *b)
 {
 	memset(b, 0, sizeof *b);
@@ -53,7 +54,7 @@ void EditorBrushInit(EditorBrush *b)
 void EditorBrushTerminate(EditorBrush *b)
 {
 	CArrayTerminate(&b->HighlightedTiles);
-	SDL_FreeSurface(b->GuideImageSurface);
+	PicFree(&b->GuideImagePic);
 }
 
 static void EditorBrushHighlightPoint(void *data, struct vec2i p)
@@ -90,8 +91,8 @@ void EditorBrushSetHighlightedTiles(EditorBrush *b)
 		}
 		break;
 	case BRUSHTYPE_BOX:
-	case BRUSHTYPE_BOX_AND_FILL:	// fallthrough
-	case BRUSHTYPE_SET_EXIT:	// fallthrough
+	case BRUSHTYPE_BOX_AND_FILL: // fallthrough
+	case BRUSHTYPE_SET_EXIT:	 // fallthrough
 		if (b->IsPainting)
 		{
 			struct vec2i v;
@@ -137,8 +138,8 @@ void EditorBrushSetHighlightedTiles(EditorBrush *b)
 			if (b->IsMoving)
 			{
 				struct vec2i v;
-				struct vec2i offset = svec2i(
-					b->Pos.x - b->DragPos.x, b->Pos.y - b->DragPos.y);
+				struct vec2i offset =
+					svec2i(b->Pos.x - b->DragPos.x, b->Pos.y - b->DragPos.y);
 				useSimpleHighlight = 0;
 				CArrayClear(&b->HighlightedTiles);
 				for (v.y = 0; v.y < b->SelectionSize.y; v.y++)
@@ -181,7 +182,8 @@ void EditorBrushSetHighlightedTiles(EditorBrush *b)
 				{
 					for (v.x = 0; v.x < b->SelectionSize.x; v.x++)
 					{
-						struct vec2i vOffset = svec2i_add(v, b->SelectionStart);
+						struct vec2i vOffset =
+							svec2i_add(v, b->SelectionStart);
 						EditorBrushHighlightPoint(b, vOffset);
 					}
 				}
@@ -199,13 +201,17 @@ void EditorBrushSetHighlightedTiles(EditorBrush *b)
 	}
 }
 
-static void SetTile(Mission *m, const struct vec2i pos, const int tile)
+static void SetTile(
+	MapBuilder *mb, Mission *m, const struct vec2i pos, const int tile)
 {
-	if (MissionStaticTrySetTile(&m->u.Static, m->Size, pos, tile))
+	if (MissionStaticTrySetTile(&m->u.Static, mb->Map->Size, pos, tile))
 	{
 		const TileClass *tc = MissionStaticGetTileClass(
-			&m->u.Static, m->Size, pos);
-		MapBuildTile(&gMap, m, pos, tc);
+			&mb->mission->u.Static, mb->Map->Size, pos);
+		if (tc != NULL)
+		{
+			MapBuildTile(mb, pos, tc);
+		}
 	}
 }
 
@@ -220,13 +226,16 @@ static void EditorBrushPaintTilesAt(void *data, struct vec2i pos)
 	EditorBrushPaintTilesAtData *paintData = data;
 	EditorBrush *b = paintData->brush;
 	Mission *m = paintData->mission;
+	MapBuilder mb;
+	MapBuilderInit(&mb, &gMap, m, GAME_MODE_NORMAL, NULL);
 	for (v.y = 0; v.y < b->BrushSize; v.y++)
 	{
 		for (v.x = 0; v.x < b->BrushSize; v.x++)
 		{
-			SetTile(m, svec2i_add(pos, v), b->PaintType);
+			SetTile(&mb, m, svec2i_add(pos, v), b->PaintType);
 		}
 	}
+	MapBuilderTerminate(&mb);
 }
 static void EditorBrushPaintLine(EditorBrush *b, Mission *m)
 {
@@ -247,6 +256,7 @@ static void EditorBrushPaintLine(EditorBrush *b, Mission *m)
 }
 typedef struct
 {
+	MapBuilder mb;
 	Mission *m;
 	int fromType;
 	int toType;
@@ -266,10 +276,10 @@ EditorResult EditorBrushStartPainting(EditorBrush *b, Mission *m, int isMain)
 		b->IsPainting = true;
 		EditorBrushPaintLine(b, m);
 		return EDITOR_RESULT_CHANGED;
-	case BRUSHTYPE_LINE:	// fallthrough
-	case BRUSHTYPE_BOX:	// fallthrough
-	case BRUSHTYPE_BOX_FILLED:	// fallthrough
-	case BRUSHTYPE_BOX_AND_FILL:	// fallthrough
+	case BRUSHTYPE_LINE:		 // fallthrough
+	case BRUSHTYPE_BOX:			 // fallthrough
+	case BRUSHTYPE_BOX_FILLED:	 // fallthrough
+	case BRUSHTYPE_BOX_AND_FILL: // fallthrough
 	case BRUSHTYPE_SET_EXIT:
 		// don't paint until the end
 		break;
@@ -297,7 +307,8 @@ EditorResult EditorBrushStartPainting(EditorBrush *b, Mission *m, int isMain)
 		// another type
 		// Don't paint if target already same type
 		// Special case: don't flood-fill doors
-		if (MissionStaticIdTileClass(&m->u.Static, b->PaintType)->Type == TILE_CLASS_DOOR)
+		if (MissionStaticIdTileClass(&m->u.Static, b->PaintType)->Type ==
+			TILE_CLASS_DOOR)
 		{
 			break;
 		}
@@ -312,11 +323,14 @@ EditorResult EditorBrushStartPainting(EditorBrush *b, Mission *m, int isMain)
 			data.Fill = MissionFillTile;
 			data.IsSame = MissionIsTileSame;
 			PaintFloodFillData pData;
+			MapBuilderInit(&pData.mb, &gMap, m, GAME_MODE_NORMAL, NULL);
 			pData.m = m;
 			pData.fromType = tile;
 			pData.toType = b->PaintType;
 			data.data = &pData;
-			if (!CFloodFill(b->Pos, &data))
+			const bool flooded = CFloodFill(b->Pos, &data);
+			MapBuilderTerminate(&pData.mb);
+			if (!flooded)
 			{
 				break;
 			}
@@ -332,7 +346,11 @@ EditorResult EditorBrushStartPainting(EditorBrush *b, Mission *m, int isMain)
 	case BRUSHTYPE_ADD_ITEM:
 		if (isMain)
 		{
-			if (MissionStaticTryAddItem(&m->u.Static, b->u.MapObject, b->Pos))
+			const Tile *tile = MapGetTile(&gMap, b->Pos);
+			const Tile *tileAbove = MapGetTile(&gMap, svec2i(b->Pos.x, b->Pos.y - 1));
+
+			if (MapObjectIsTileOK(b->u.MapObject, tile, tileAbove) &&
+				MissionStaticTryAddItem(&m->u.Static, b->u.MapObject, b->Pos))
 			{
 				return EDITOR_RESULT_CHANGED_AND_RELOAD;
 			}
@@ -348,8 +366,14 @@ EditorResult EditorBrushStartPainting(EditorBrush *b, Mission *m, int isMain)
 	case BRUSHTYPE_ADD_CHARACTER:
 		if (isMain)
 		{
-			if (MissionStaticTryAddCharacter(
-				&m->u.Static, b->u.ItemIndex, b->Pos))
+			const Tile *tile = MapGetTile(&gMap, b->Pos);
+			if (TileIsClear(tile))
+			{
+				CharacterPlace cp = {b->Pos, DIRECTION_DOWN};
+				MissionStaticAddCharacter(&m->u.Static, b->u.ItemIndex, cp);
+				return EDITOR_RESULT_CHANGED_AND_RELOAD;
+			}
+			else if (MissionStaticTryRotateCharacterAt(&m->u.Static, b->Pos))
 			{
 				return EDITOR_RESULT_CHANGED_AND_RELOAD;
 			}
@@ -365,15 +389,13 @@ EditorResult EditorBrushStartPainting(EditorBrush *b, Mission *m, int isMain)
 	case BRUSHTYPE_ADD_OBJECTIVE:
 		if (isMain)
 		{
-			if (MissionStaticTryAddObjective(
-				&m->u.Static, b->u.ItemIndex, b->Index2, b->Pos))
-			{
-				return EDITOR_RESULT_CHANGED_AND_RELOAD;
-			}
+			MissionStaticAddObjective(
+				m, &m->u.Static, b->u.ItemIndex, b->Index2, b->Pos, false);
+			return EDITOR_RESULT_CHANGED_AND_RELOAD;
 		}
 		else
 		{
-			if (MissionStaticTryRemoveObjectiveAt(&m->u.Static, b->Pos))
+			if (MissionStaticTryRemoveObjective(m, &m->u.Static, b->Pos))
 			{
 				return EDITOR_RESULT_CHANGED_AND_RELOAD;
 			}
@@ -382,8 +404,10 @@ EditorResult EditorBrushStartPainting(EditorBrush *b, Mission *m, int isMain)
 	case BRUSHTYPE_ADD_KEY:
 		if (isMain)
 		{
-			if (MissionStaticTryAddKey(&m->u.Static, b->u.ItemIndex, b->Pos))
+			const Tile *tile = MapGetTile(&gMap, b->Pos);
+			if (TileIsClear(tile))
 			{
+				MissionStaticAddKey(&m->u.Static, b->u.ItemIndex, b->Pos);
 				return EDITOR_RESULT_CHANGED_AND_RELOAD;
 			}
 		}
@@ -395,11 +419,29 @@ EditorResult EditorBrushStartPainting(EditorBrush *b, Mission *m, int isMain)
 			}
 		}
 		break;
+	case BRUSHTYPE_ADD_PICKUP:
+		if (isMain)
+		{
+			const Tile *tile = MapGetTile(&gMap, b->Pos);
+			if (TileCanWalk(tile) && MissionStaticTryAddPickup(
+					&m->u.Static, b->u.Pickup, b->Pos))
+			{
+				return EDITOR_RESULT_CHANGED_AND_RELOAD;
+			}
+		}
+		else
+		{
+			if (MissionStaticTryRemovePickupAt(&m->u.Static, b->Pos))
+			{
+				return EDITOR_RESULT_CHANGED_AND_RELOAD;
+			}
+		}
+		break;
 	case BRUSHTYPE_SET_KEY:
-		if (isMain || b->u.ItemIndex > 0)
+		if (isMain)
 		{
 			if (MissionStaticTrySetKey(
-				&m->u.Static, b->u.ItemIndex, m->Size, b->Pos))
+					&m->u.Static, b->u.ItemIndex, m->Size, b->Pos))
 			{
 				return EDITOR_RESULT_CHANGED_AND_RELOAD;
 			}
@@ -422,12 +464,18 @@ EditorResult EditorBrushStartPainting(EditorBrush *b, Mission *m, int isMain)
 static void MissionFillTile(void *data, const struct vec2i v)
 {
 	PaintFloodFillData *pData = data;
-	SetTile(pData->m, v, pData->toType);
+	SetTile(&pData->mb, pData->m, v, pData->toType);
 }
 static bool MissionIsTileSame(void *data, const struct vec2i v)
 {
 	PaintFloodFillData *pData = data;
-	return MissionStaticGetTile(&pData->m->u.Static, pData->m->Size, v) == pData->fromType;
+	if (!Rect2iIsInside(Rect2iNew(svec2i_zero(), pData->mb.Map->Size), v))
+	{
+		return false;
+	}
+	return MissionStaticGetTile(
+			   &pData->mb.mission->u.Static, pData->mb.Map->Size, v) ==
+		   pData->fromType;
 }
 static void EditorBrushPaintBox(
 	EditorBrush *b, Mission *m, const int lineType, const int fillType)
@@ -438,8 +486,7 @@ static void EditorBrushPaintBox(
 	// aware of brush sizes
 	struct vec2i v;
 	struct vec2i d = svec2i(
-		b->Pos.x > b->LastPos.x ? 1 : -1,
-		b->Pos.y > b->LastPos.y ? 1 : -1);
+		b->Pos.x > b->LastPos.x ? 1 : -1, b->Pos.y > b->LastPos.y ? 1 : -1);
 	EditorBrushPaintTilesAtData paintData;
 	paintData.brush = b;
 	paintData.mission = m;
@@ -511,10 +558,10 @@ EditorResult EditorBrushStopPainting(EditorBrush *b, Mission *m)
 				// Copy tiles to temp from selection, clearing them
 				// in the process
 				RECT_FOREACH(Rect2iNew(b->SelectionStart, b->SelectionSize))
-					const int tile =
-						MissionStaticGetTile(&m->u.Static, m->Size, _v);
-					CArrayPushBack(&movedTiles, &tile);
-					MissionStaticClearTile(&m->u.Static, m->Size, _v);
+				const int tile =
+					MissionStaticGetTile(&m->u.Static, m->Size, _v);
+				CArrayPushBack(&movedTiles, &tile);
+				MissionStaticClearTile(&m->u.Static, m->Size, _v);
 				RECT_FOREACH_END()
 				// Move the selection to the new position
 				b->SelectionStart.x += b->Pos.x - b->DragPos.x;
@@ -522,14 +569,13 @@ EditorResult EditorBrushStopPainting(EditorBrush *b, Mission *m)
 				// Copy tiles to the new area, for parts of the new area that
 				// are valid
 				RECT_FOREACH(Rect2iNew(b->SelectionStart, b->SelectionSize))
-					if (_v.x >= 0 && _v.x < m->Size.x &&
-						_v.y >= 0 && _v.y < m->Size.y)
-					{
-						const int tile = *(int *)CArrayGet(&movedTiles, _i);
-						MissionStaticTrySetTile(
-							&m->u.Static, m->Size, _v, tile);
-						result = EDITOR_RESULT_CHANGED_AND_RELOAD;
-					}
+				if (_v.x >= 0 && _v.x < m->Size.x && _v.y >= 0 &&
+					_v.y < m->Size.y)
+				{
+					const int tile = *(int *)CArrayGet(&movedTiles, _i);
+					MissionStaticTrySetTile(&m->u.Static, m->Size, _v, tile);
+					result = EDITOR_RESULT_CHANGED_AND_RELOAD;
+				}
 				RECT_FOREACH_END()
 				CArrayTerminate(&movedTiles);
 				// Update the selection to fit within map boundaries
@@ -576,31 +622,34 @@ EditorResult EditorBrushStopPainting(EditorBrush *b, Mission *m)
 				}
 			}
 			break;
-		case BRUSHTYPE_SET_EXIT:
+		case BRUSHTYPE_SET_EXIT: {
+			struct vec2i exitStart = svec2i_min(b->LastPos, b->Pos);
+			struct vec2i exitEnd = svec2i_max(b->LastPos, b->Pos);
+			// Clamp within map boundaries
+			exitStart = svec2i_clamp(
+				exitStart, svec2i_zero(),
+				svec2i_subtract(m->Size, svec2i_one()));
+			exitEnd = svec2i_clamp(
+				exitEnd, svec2i_zero(),
+				svec2i_subtract(m->Size, svec2i_one()));
+			// Check that size is big enough
+			const struct vec2i size = svec2i_subtract(exitEnd, exitStart);
+			if (size.x < 2 || size.y < 2)
 			{
-				struct vec2i exitStart = svec2i_min(b->LastPos, b->Pos);
-				struct vec2i exitEnd = svec2i_max(b->LastPos, b->Pos);
-				// Clamp within map boundaries
-				exitStart = svec2i_clamp(
-					exitStart, svec2i_zero(), svec2i_subtract(m->Size, svec2i_one()));
-				exitEnd = svec2i_clamp(
-					exitEnd, svec2i_zero(), svec2i_subtract(m->Size, svec2i_one()));
-				// Check that size is big enough
-				struct vec2i size =
-					svec2i_add(svec2i_subtract(exitEnd, exitStart), svec2i_one());
-				if (size.x >= 3 && size.y >= 3)
-				{
-					// Check that exit area has changed
-					if (!svec2i_is_equal(exitStart, m->u.Static.Exit.Start) ||
-						!svec2i_is_equal(exitEnd, m->u.Static.Exit.End))
-					{
-						m->u.Static.Exit.Start = exitStart;
-						m->u.Static.Exit.End = exitEnd;
-						result = EDITOR_RESULT_CHANGED_AND_RELOAD;
-					}
-				}
+				break;
 			}
-			break;
+			// Check that exit area has changed
+			Exit *exit = CArrayGet(&m->u.Static.Exits, b->u.ItemIndex);
+			if (svec2i_is_equal(exitStart, exit->R.Pos) &&
+				svec2i_is_equal(size, exit->R.Size))
+			{
+				break;
+			}
+			exit->R.Pos = exitStart;
+			exit->R.Size = size;
+			result = EDITOR_RESULT_CHANGED_AND_RELOAD;
+		}
+		break;
 		default:
 			// do nothing
 			break;
@@ -610,3 +659,16 @@ EditorResult EditorBrushStopPainting(EditorBrush *b, Mission *m)
 	CArrayClear(&b->HighlightedTiles);
 	return result;
 }
+
+bool EditorBrushTryLoadGuideImage(EditorBrush *b, const char *filename)
+{
+	PicFree(&b->GuideImagePic);
+	SDL_Surface *s = IMG_Load(filename);
+	if (s == NULL)
+		return false;
+	SDL_Surface *sc = SDL_ConvertSurface(s, gGraphicsDevice.Format, 0);
+	SDL_FreeSurface(s);
+	PicLoad(&b->GuideImagePic, svec2i(sc->w, sc->h), svec2i_zero(), sc);
+	SDL_FreeSurface(sc);
+	return true;
+}
